// =========================================================================================================
//  Mobile Target Range
//  Version: 2.1.0 (ArmA)
//  Author: Kronzky (www.kronzky.info / kronzky@gmail.com)
//
// =========================================================================================================
//
//  Creates renewable targets at a defined range, and provides ballistic information
//
// =========================================================================================================
//
//  Mandatory Parameters:
//                   nul=[] exec "targetrange.sqf"
//
//  Optional Parameters:
//                nul=[[targets],mindist,maxdist,stepsize,movespeed,movedist] execVM "targetrange.sqf"
//          e.g.  nul=[["SoldierEB",0,0,"T72"],"min:50","max:300"] execVM "targetrange.sqf"
//
//     targets  : Array of targets to be used. 
//                Array entries can be either objects that have already been placed on the map,
//                a class name, or a zero (in that case a gap is left at that position).
//
//     mindist  : The closest distance for the range
//     (50)       Enter as "min:<value>", e.g. [...,"min:100",...]
//                   
//     maxdist  : The furthest distance for the range
//     (500)      Enter as "max:<value>", e.g. [...,"max:500",...]    
//                   
//     stepsize : The distance between range options/markers
//     (50)       Enter as "step:<value>", e.g. [...,"step:100",...]   
//                   
//     movespeed: If > 0 then targets will move left and right (1=slowest, 10=fastest)
//     (0)        Enter as "speed:<value>", e.g. [...,"speed:2",...]     
//                   
//     movedist : How far moving targets should move
//     (0)        Enter as "move:<value>", e.g. [...,"move:10",...]     
//                 
//     Only the position of the target array is critical (has to be the first argument).
//     All other options can be defined in any position.
//  
// =========================================================================================================
//
//  After each shot the distance reached (and average speed) is displayed in the lower left corner.
//  To access the "bullet cam" (which follows the fired projectile), or slow-mo mode, use the radio menu.
//
// =========================================================================================================
#include "KRON_Strings.sqf";

sleep .1;
KRON_orgPos=getPos player;
KRON_orgDir=getDir player;
KRON_space=3;
KRON_SlowMo=1;

KRON_rotpoint = {private["_cp","_a","_tx","_ty","_cd","_sd","_cx","_cy","_xout","_yout"];_cp=_this select 0; _cx=_cp select 0; _cy=_cp select 1; _a=_this select 1; _cd=cos(_a*-1); _sd=sin(_a*-1); _tx=_this select 2; _ty=_this select 3; _xout=if (_a!=0) then {_cx+ (_cd*_tx - _sd*_ty)} else {_cx+_tx}; _yout=if (_a!=0) then {_cy+ (_sd*_tx + _cd*_ty)} else {_cy+_ty}; [_xout,_yout,0]};
KRON_addangle = {private["_a","_d"]; _a=_this select 0; _d=_this select 1; _a=_a+_d; if (_a<0) then {_a=_a+360}; _a=_a mod 360; _a};
KRON_makeobj = {private["_o","_p","_x","_y","_z","_d","_o1"];_o=_this select 0; _p=_this select 1; _x=_p select 0; _y=_p select 1; _z=_p select 2; _d=_this select 2; _o1 = _o createVehicle [_x,_y,_z]; _o1 setDir _d; _o1 setPos [_x,_y,_z]; _o1 setVectorUp [0,0,1]; _o1};
KRON_xOff={private["_i","_s","_dx"];_i=(_this select 0); _s=_this select 1;	_dx=(_i*_s)-(_s*((count KRON_objects)-1)/2); _dx;};
KRON_makemarker = {private["_p","_x","_y","_mrk"];_p=_this select 0; _x=_p select 0; _y=_p select 1; _mrk=format["mrk_%1_%2_%3",random 1000000,_x,_y]; createMarker[_mrk,[_x,_y]];	_mrk setMarkerType "DOT"; _mrk setMarkerSize [.5,.5];	};
KRON_weapondir = {private["_g","_v","_a"];if (count _this==0) then {{_a=((vehicle player) weaponDirection _x) select 2; if (_a!=0) then {_g=_x}}forEach weapons (vehicle player)} else {_g = _this select 0;}; _a=floor(90*(((vehicle player) weaponDirection _g) select 2)*100)/100;_a;};
KRON_isSubClass = {private["_cs","_cp","_is","_par"];	_cs = _this select 0; _cp = _this select 1; _is=0; while {_is==0} do {_par=configName (inheritsFrom (configFile >> "CfgVehicles" >> _cs)); switch (_par) do {case _cp: {_is=1}; case "All": {_is=-1}; default {_cs=_par};};}; (_is==1);};

// re-position targets
_exit=false;
_move=false;
if (count _this>0) then {
	if (typeName (_this select 0)=="OBJECT") then {_move=true};
};

KRON_moved=false;
if (_move) then {
	_args = _this select 3;
	KRON_dist = _args select 0;
	titleText [format["\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nCurrent distance: %1 meters",KRON_dist],"plain",9999];
	for "_i" from 0 to (count KRON_targets)-1 do {
		if (typename (KRON_targets select _i)=="OBJECT") then {
			_pos=[KRON_orgPos, KRON_orgDir, [_i,KRON_space] call KRON_xOff, KRON_dist] call KRON_rotpoint;
			_obj=KRON_targets select _i;
			_obj setPos [_pos select 0, _pos select 1, 0];
			_obj setDir ([KRON_orgDir,(KRON_objects select _i) select 1] call KRON_addangle);
		};
	};
	[] call KRON_makeborders;	
	KRON_moved=true;
	_exit=true;
} else {
	KRON_mindist = parseNumber([_this,"min","50"] call KRON_getArg);
	KRON_maxdist = parseNumber([_this,"max","500"] call KRON_getArg);
	KRON_step = parseNumber([_this,"step","50"] call KRON_getArg);
	KRON_objects = [];
	if ((typeName (_this select 0))=="array") then {
		{
			if (typeName _x=="object") then {
					KRON_objects = KRON_objects + [[typeOf _x,direction _x]];
					deleteVehicle _x;
			} else {
				_dir=0;
				if (typeName _x=="string") then {
					if ([_x,"Man"] call KRON_isSubClass) then {_dir=180};
				};
				KRON_objects = KRON_objects + [[_x,_dir]];
			};
		}forEach (_this select 0);
	};
	if (count KRON_objects == 0) then {KRON_objects = [["TargetEpopup",0]]};
	KRON_Cnt=0;
};
if (_exit) exitWith {};
if ((typeName KRON_objects!="array") || (count KRON_array<1)) exitWith {hint "No targets defined"};

_movespeed = parseNumber([_this,"speed","0"] call KRON_getArg);
KRON_Move = (_movespeed>0);
_movedist = parseNumber([_this,"move","10"] call KRON_getArg);

KRON_dist=KRON_mindist;

// create a target object	
KRON_makeobject = {
	_idx=_this select 0;
	_pos=[KRON_orgPos, KRON_orgDir, [_idx,KRON_space] call KRON_xOff, KRON_dist] call KRON_rotpoint;
	_unit=0;
	KRON_Cnt=KRON_Cnt+1;
	if ([(KRON_objects select _idx) select 0,"Man"] call KRON_isSubClass) then {
		call compile format["""%1"" createUnit [_pos, KRON_grp,""KRON_U_%2 = this"", 0.1, ""private""]",(KRON_objects select _idx) select 0,KRON_Cnt];
		call compile format["_unit=KRON_U_%1",KRON_Cnt];
		_unit setPos [_pos select 0, _pos select 1, .5];
		{_unit removeMagazine _x} forEach magazines _unit;
		_unit setBehaviour "careless"; 
		_unit allowFleeing 0; 
		_unit disableAI "MOVE";
		_unit disableAI "TARGET";
		_unit doWatch player;
		_unit setUnitPos (KRON_unitpos select _idx);
	} else {
		if (isClass(configFile >> "cfgVehicles" >> (KRON_objects select _idx) select 0)) then {
			_unit = [(KRON_objects select _idx) select 0,_pos,(KRON_objects select _idx) select 1] call KRON_makeobj;
			_unit setVehicleVarName format["KRON_U_%1",KRON_Cnt]; 
			_unit call compile format["KRON_U_%1=_unit",KRON_Cnt];
		};
	};
	if (typename _unit=="OBJECT") then {
		//KRON_dirs set [_idx,(KRON_objects select _idx) select 1];
		_unit setDir ([KRON_orgDir,(KRON_objects select _idx) select 1] call KRON_addangle);
		KRON_targets set [_idx,_unit];
		_unit addEventHandler["hit","KRON_HQ globalchat format['""%1"" hit, damage:%2',getText(configFile >> 'cfgVehicles' >> typeof (_this select 0) >> 'displayName'),floor((_this select 2)*100)/100]; "]; 
		call compile format["_unit addEventHandler[""killed"",""setAccTime .99; KRON_HQ globalchat '""""%2"""" killed'; deleteVehicle (_this select 0)""]",_idx,getText(configFile >> "cfgVehicles" >> typeof _unit >> "displayName")];
		_trgname=format["KRON_Trig_%1",KRON_Cnt];
		call compile format["%1=createTrigger[""EmptyDetector"",[0,0]]",_trgname];
		call compile format["%1 setTriggerArea[0,0,0,true]",_trgname];
		call compile format["%1 setTriggerActivation[""NONE"",""PRESENT"",false]",_trgname];
		call compile format["%1 setTriggerTimeout[3,3,3,false]",_trgname];
		call compile format["%1 setTriggerStatements[""!(alive KRON_U_%3)"", ""[%2] call KRON_makeobject; setAccTime KRON_SlowMo; deleteVehicle KRON_Trig_%3"", """"]",_trgname,_idx,KRON_Cnt];
	};
	setAccTime KRON_SlowMo;
	_unit;
};	

// create distance markers
KRON_walls=[];
KRON_makeborders = {
	if (count KRON_borders!=0) then {
		{deleteVehicle _x}forEach KRON_borders;
		KRON_borders=[];
		};
	_off=([count KRON_objects,KRON_space] call KRON_xOff)+5;
	_doff=3;
	for [{_d=KRON_mindist-_doff},{_d<=KRON_maxdist-_doff},{_d=_d+KRON_step}] do {
		_x=0;_z=0; 
		if ((_d mod 100)!=0) then {_x=1;_z=-1};
		_pos=[KRON_orgPos, KRON_orgDir, _x-_off-2.5 , _d] call KRON_rotpoint;
		_o=["Wall1",[_pos select 0,_pos select 1,_z],KRON_orgDir] call KRON_makeobj;
		KRON_walls=KRON_walls+[_o];
		KRON_borders=KRON_borders+[_o];
		_pos=[KRON_orgPos, KRON_orgDir, _x-_off-5 , _d] call KRON_rotpoint;
		_o=["Wall1",[_pos select 0,_pos select 1,_z],KRON_orgDir] call KRON_makeobj;
		KRON_borders=KRON_borders+[_o];
		_pos=[KRON_orgPos, KRON_orgDir, _off-_x, _d] call KRON_rotpoint;
		_o=["Wall1",[_pos select 0,_pos select 1,_z],KRON_orgDir] call KRON_makeobj;
		KRON_walls=KRON_walls+[_o];
		KRON_borders=KRON_borders+[_o];
		_pos=[KRON_orgPos, KRON_orgDir, _off-_x+2.5, _d] call KRON_rotpoint;
		_o=["Wall1",[_pos select 0,_pos select 1,_z],KRON_orgDir] call KRON_makeobj;
		KRON_borders=KRON_borders+[_o];
	};
	for [{_d=(floor(KRON_mindist/100))},{_d<=(floor(KRON_maxdist/100))},{_d=_d+1}] do {
		for "_i" from 1 to _d do {
			_pos=[KRON_orgPos, KRON_orgDir, .5-_i-_off, _d*100+.5] call KRON_rotpoint;
			_o=["FlagCarrierNorth",_pos,0] call KRON_makeobj;
			KRON_borders=KRON_borders+[_o];
			KRON_walls=KRON_walls+[_o];
		};
	};
};

// create initial targets
_opfor=switch (side player) do {case west:{east}; case east:{west}; default {east}};
createCenter _opfor;
KRON_grp=createGroup _opfor;
KRON_targets=[];
KRON_borders=[];
KRON_unitpos=[];
KRON_markers=[];
//KRON_dirs=[];
_stances=["up","middle","down"]; 
_stanceidx=0;

for "_i" from 0 to (count KRON_objects)-1 do {
	//KRON_dirs set [_i,180];
	if (typeName ((KRON_objects select _i) select 0)=="STRING") then {
		if ([(KRON_objects select _i) select 0,"Man"] call KRON_isSubClass) then {
			KRON_unitpos set [_i,_stances select _stanceidx];
			_stanceidx=_stanceidx+1;
			if (_stanceidx>=count _stances) then {_stanceidx=0};
		} else {
			KRON_unitpos set [_i,0];
			_isvehicle=false;
		};
		_obj=[_i] call KRON_makeobject;
	} else {
		KRON_targets set [_i,0]; 
	};
};

// create action menu
for [{_d=KRON_mindist},{_d<=KRON_maxdist},{_d=_d+KRON_step}] do {
	call compile format["player addAction [""Distance: %1m"", ""targetrange.sqf"", [%1]]",_d];
};
titleText["\n\n\n\n\n\n\nUse action menu to change distance","plain"];

player addAction ["Select Weapon", "selectweapon.sqf"];

// create radio calls
KRON_BulletCam=0;
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Alpha","PRESENT",true];
_radio setTriggerStatements["this","KRON_BulletCam=0;",""];
1 setRadioMsg "BulletCam off";
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Bravo","PRESENT",true];
_radio setTriggerStatements["this","KRON_BulletCam=1.00;",""];
2 setRadioMsg "BulletCam on (acc: 1.00)";
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Charlie","PRESENT",true];
_radio setTriggerStatements["this","KRON_BulletCam=.10;",""];
3 setRadioMsg "BulletCam on (acc: 0.10)";
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Delta","PRESENT",true];
_radio setTriggerStatements["this","KRON_BulletCam=.01;",""];
4 setRadioMsg "BulletCam on (acc: 0.01)";
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Echo","PRESENT",true];
_radio setTriggerStatements["this","if (KRON_SlowMo>.5) then {5 setRadioMsg 'Slow-Mo off'; setAccTime .1; KRON_SlowMo=.1} else {5 setRadioMsg 'Slow-Mo on'; setAccTime 1; KRON_SlowMo=1};",""];
5 setRadioMsg 'Slow-Mo on';
KRON_Marker=false;
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Foxtrot","PRESENT",true];
_radio setTriggerStatements["this","6 setRadioMsg (if (KRON_Marker) then {'Destination marker off'} else {'Destination marker on'}); KRON_Marker=!(KRON_Marker);",""];
6 setRadioMsg "Destination marker off";
KRON_Ballistics=false;
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Golf","PRESENT",true];
_radio setTriggerStatements["this","hint ''; 7 setRadioMsg (if (KRON_Ballistics) then {'Ballistics on'} else {'Ballistics off'}); KRON_Ballistics=!(KRON_Ballistics);",""];
7 setRadioMsg "Ballistics on";
_ballistics=createTrigger["EmptyDetector",[0,0]];
_ballistics setTriggerActivation["NONE","PRESENT",true];
_ballistics setTriggerStatements["format['%1',ballloop]!='false'","wdir=[] call KRON_weapondir; ballloop=false; if (KRON_Ballistics && !dialog) then {hint format['elev: %1',wdir]}","ballloop=true"];
if (_movespeed>0) then {
	_radio=createTrigger["EmptyDetector",[0,0]];
	_radio setTriggerActivation["Hotel","PRESENT",true];
	_radio setTriggerStatements["this","8 setRadioMsg (if (KRON_Move) then {'Movement on'} else {'Movement off'}); KRON_Move=!(KRON_Move);",""];
	8 setRadioMsg "Movement off";
};

//if (true) exitWith {hint format["obj:%1",count KRON_targets]};
KRON_dirshot=999;
[player,0,0,[KRON_mindist]] execVM "targetrange.sqf";
player addEventHandler["fired","setAccTime .001; fired=_this select 1; KRON_dirshot=floor(90*((player weaponDirection fired) select 2)*100)/100; hint ''; _this execVM 'bullettrack.sqf'"];
{_x addEventHandler["fired","setAccTime .001; fired=_this select 1; KRON_dirshot=[fired] call KRON_weapondir; hint ''; _this execVM 'bullettrack.sqf'"]}forEach vehicles;
KRON_HQ="Logic" createVehicle [0,0];

_movestep=.01*_movespeed;
_orgpos=[];
_moveoff=0;
_movedir=1;
_pdiff=0;
_cycles=0;
KRON_moved=true;
K_op=position (KRON_targets select 0);
while {true} do {
	if (_movespeed>0) then {
		if (KRON_moved) then {
			for "_u" from 0 to (count KRON_targets)-1 do {
				_unit = KRON_targets select _u;
				if (typename _unit=="object") then {
					_orgpos set [_u,getPos _unit];
				};
			};
			_moveoff=0;
			_movedir=1;
			_pdiff=0;
			KRON_moved=false;
		};
		if (KRON_Move) then {
			_moveoff=_moveoff+_movedir*_movestep;
			if (abs(_moveoff)>_movedist/2)	then {_movedir=_movedir*-1};
			_pdiff=_pdiff+(_movedir*_movestep);
		};
		for "_u" from 0 to (count KRON_targets)-1 do {
			_unit = KRON_targets select _u;
			if (typename _unit=="object") then {
				_pos=[_orgpos select _u, KRON_orgDir, _pdiff, 0] call KRON_rotpoint;
				_unit setPos _pos;
			}
		};
	};
	sleep .01;
	_cycles=_cycles+.01;
	
	if (_cycles>.5) then {
		_muzzles = [];
	  _weapons = weapons (vehicle player);
	  _ammo = 10000;
	  {
	  	_weaponMuzzles = getArray (configFile >> "CfgWeapons" >> _x >> "muzzles");
	    if ((_weaponMuzzles select 0)=="this") then {
				_muzzles = _muzzles + [_x];
	    } else {
				_muzzles = _muzzles + _weaponMuzzles;
			};
		}forEach _weapons;
	  (vehicle player) setvehicleammo 1;
		{
			_mxammo = (vehicle player) ammo _x;
	    if (_mxammo<_ammo && _mxammo>0) then {
	    	_ammo=((vehicle player) ammo _x);
	    };
		}forEach _muzzles;
		{player reveal _x}forEach KRON_walls;
		_cycles=0;
	};	
};
